/************************************************************************/
/*	Author: 	Nate Eastland*/
/*	Copyright (c) 2017, Digilent Inc, All rights reserved		        */
/************************************************************************/
/*  Module Description: 												*/
/*																		*/
/*		This is for the MMC34166 magnetic sensor of the PmodCMPS2.							    */
/*																		*/
/************************************************************************/
/*  	Revision History:												*/
/*                                                                      */
/*
        1/4/2017    (NateE) created for component MMC3416xPJ*/
/*																		*/
/************************************************************************/

/************************************************************************/
/*                                                                      */
/*  This library is free software; you can redistribute it and/or       */
/*  modify it under the terms of the GNU Lesser General Public          */
/*  License as published by the Free Software Foundation; either        */
/*  version 2.1 of the License, or (at your option) any later version.  */
/*                                                                      */
/*  This library is distributed in the hope that it will be useful,     */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of      */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   */
/*  Lesser General Public License for more details.                     */
/*                                                                      */
/*  You should have readd a copy of the GNU Lesser General Public    */
/*  License along with this library; if not, write to the Free Software */
/*  Foundation, Inc.,51 Franklin St, Fifth Floor, Boston, MA  02110-1301*/
/*  USA                                                                 */
/************************************************************************/


#ifndef CMPS2_H
#define CMPS2_H

#include "Wire.h"
#include "math.h"

//chip measurement and status register
const uint8_t statusReg = 0x06;

//7-bit device address; format of 0x0110xxx, with lower three determined per device

/*---------------------------------------------------------------------------------------*/
/*                     Internal control/configuration registers                          */
/*                                                                                       */
/*intConReg0		0x07 bit field --> [7] refil cap, [6] reset, [5] set, [4] no boost,          */
/*                  [3] CM Freq1, [2] CM Freq2, [1] Cont mode, [0] Take measurement      */
/*
/* *CM Freq0 and Freq1 used to determine sample frequency in continuous measurement mode
/*	 Freq0	 Freq1	 Meas. Freq (Hz)
/*	   0	    0	       1.5
/*     0	    1	       13
/*     1	    0	       25
/*     1	    1	       50
/*
/* intConReg1		0x08 bit field --> [7] SW reset, [6] temp test, [5] ST_XYZ, [4:2] Z,Y,X inhibit,
/*			[1:0] BW1,BW0 used for operation mode config. (output resolution)
/*			- 00 = 16-bit mode, 7.92 ms measurement time
/*			- 01 = 16-bit mode, 4.08 ms measurement time
/*			- 10 = 14-bit mode, 2.16 ms measurement time
/*			- 11 = 12-bit mode, 1.20 ms measurement time
/*--------------------------------------------------------------------------------------------*/
const uint8_t intConReg0 = 0x07;
const uint8_t intConReg1 = 0x08;

const uint8_t readReady = 0x01; //just a constant used in determining if device is ready to be read.

class CMPS2
{
  public:
    CMPS2();
    void init(); //set up I2C and control registers
    void removeOffset(); //procedure to remove measurement offset as a result of temp drift
    int getMeasurement();

    int degree();
    int readHeading();

    //getters
    double getOffset();
    int getX();
    int getY();
    int getZ();
    int getT1();
    int getT2();

    //setters
    void setOffset(double setVal);
    void setDevID(uint8_t newID);
    void setDeclination(int newDec);

  private:
    bool measDone;
    double offset;
    int declination;

    uint8_t deviceID;

    //byte measurement values; 16 bits of measuremnt data per coordinate
    uint8_t xLow;
    uint8_t xHigh;
    uint8_t yLow;
    uint8_t yHigh;
    uint8_t zLow;
    uint8_t zHigh;

    int x;
    int y;
    int z;

    int xAdj;
    int yAdj;
    int zAdj;

    int xMin;
    int xMax;
    int yMin;

    int yMax;
    int zMin;
    int zMax;

    int xMid;
    int yMid;
    int zMid;

    int test1;
    int test2;
};

#endif
